require "plutora_rest/version"
require "rubygems"
require 'rest-client'
require 'json'
require 'active_support/core_ext'
require 'pp'
require 'plutora_rest/sections/changes.rb'
require 'plutora_rest/sections/lookupfields.rb'
require 'plutora_rest/sections/releases.rb'
require 'plutora_rest/sections/systems.rb'

module PlutoraRest
  class Calls
  	
    attr_reader :token

    #---- Initialize ----
  	def initialize(opts = {})
  	 #- Use to initilze the object and connect to plutora
     # => token
     #
      oauth_url = opts[:oauthURL] || 'https://usoauth.plutora.com/oauth/token'

      params = {'grant_type' => 'password', 
        'client_id' => opts[:client_id], 
        'client_secret' => opts[:client_secret], 
        'username' => opts[:username], 
        'password' => opts[:password]}

      headers = {'Accept' => 'application/json', 
        'Content-Type' => 'application/x-www-form-urlencoded'}

      @base_url= opts[:baseURL] || "https://usapi.plutora.com/"

      rConnect=plutora_post({:url => oauth_url, :params => params, :headers => headers})
      rConnect=JSON.parse(rConnect)

      @token=rConnect['access_token']
      @expires_in=rConnect['expires_in']
      @expiresAt=DateTime.now + @expires_in.to_i.seconds
      @refresh_token=rConnect['refresh_token']

      @oauthURL=oauth_url
      @clientID=opts[:client_id]
      @clientSecret=opts[:client_secret]

      return @token
    end

    #---- Refresh Token If Expired ----
    def refresh_token_if_expired
      begin
        if token_expired?
          params={'grant_type' => 'refresh_token',
            'refresh_token' => @refresh_token,
            'client_id' => @clientID,
            'client_secret' => @clientSecret
          }

          headers = {'Accept' => 'application/json', 
            'Content-Type' => 'application/x-www-form-urlencoded'
          }

          refreshConnect=plutora_post({:url => @oauthURL, :params => params, :headers => headers})
          refreshConnect=JSON.parse(refreshConnect)

          @token=refreshConnect['access_token']
          @expires_in=refreshConnect['expires_in']
          @expiresAt=DateTime.now + @expires_in.to_i.seconds
          @refresh_token=refreshConnect['refresh_token']

          return @token
        end
      rescue
        puts "<----- Refresh Token If Expired ----->"
        puts $!
        puts $@
        puts "<------------------------------------>"
      end
    end

    #---- Token Expired? ----
    def token_expired?
      expiry=Time.at(@expiresAt)
      return true if expiry < Time.now
      false
    end

    #---- POST REST Call ----
    def plutora_post(opts)
      resp=RestClient.post(opts[:url], opts[:params], opts[:headers])
      return resp
    end

    def pluotra_post_json(opts = {})
      body=opts[:params].to_json
      puts body
      return RestClient.post(opts[:url], body, opts[:headers])
      # return RestClient.post(opts[:url], "{}", opts[:headers])
    end

    #---- Get REST Call ----
    def plutora_get(opts)
      resp=RestClient.get(opts[:url], opts[:headers])
      return resp
    end

    #---- PUT REST Call ----
    def plutora_put(opts)
      resp=RestClient.put(opts[:url], opts[:params], opts[:headers])
      return resp
    end

    #---- Format Response into JSON ----
    def formatResp(opts)
      json_convert=Hash.from_xml(opts[:xml]).to_json
      return JSON.parse(json_convert)
    end

  end
end
