# ----------------------------------------------
# - Section: Changes
# ----------------------------------------------

module PlutoraRest
  class Calls
  	
  	#--------------------------
  	#-------- GET CALL --------
  	#--------------------------

  	#---- Get Changes ----
  	def get_Changes(opts = {})
  		vRetry=0
  		begin
	  		token=opts[:token] || @token

	  		url = "#{@base_url}Changes"

	  		headers = {'Authorization' => 'Bearer ' + token}

	  		resp=plutora_get({:url => url, :headers => headers})
	  		resp=formatResp({:xml => resp})
	  		return resp
	  	rescue
	  		puts "-----> Get Changes Error <-----"
	  		puts $!
	  		puts $@
	  		puts "<----------------------------->"
	  		if $! == "500 Internal Server Error"
	          vRetry+=1
	          retry if vRetry < 2
	        end
	  	end
  	end

  	#---- Get Changes By Id ----
  	def get_ChangesById(opts = {})
  		vRetry=0
  		begin
	  		if opts[:id] == nil
	  			raise "Changes By Id requires an ID value!"
	  		end

	  		url = "#{@base_url}Changes/#{opts[:id]}"

	  		headers = {'Authorization' => 'Bearer ' + (opts[:token] == nil ? @token : opts[:token])}

	  		resp=formatResp({:xml => plutora_get({:url => url, :headers => headers})})
	  		return resp
	  	rescue
	  		puts "-----> Get Changes By Id Error <-----"
	  		puts $!
	  		puts $@
	  		puts "<----------------------------------->"
	  		if $! == "500 Internal Server Error"
	          vRetry+=1
	          retry if vRetry < 2
	        end
	  	end
  	end

  	#---- Get Changes By Id Additional Informations ----
  	def get_ChangesById_AdditionalInformation(opts = {})
  		if opts[:id] == nil
  			raise "Changes By Id - Additional Information requires an ID value!"
  		end

  		url = "#{@base_url}Changes/#{opts[:id]}/additionalInformation"

  		headers = {'Authorization' => "Bearer #{(opts[:token] == nil ? @token : opts[:token])}"}

		resp=formatResp({:xml => plutora_get({:url => url, :headers => headers})})
		return resp
  	end

  	#---- Get Changes by Filter ----
  	def get_ChangesFilter(opts = {})
  		if opts[:searchfilter] == nil
  			raise "Change by Filter - Search Filters required!"
  		end

  		headers = {'Authorization' => "Bearer #{(opts[:token] == nil ? @token : opts[:token])}",
        	'Content-Type' => 'application/json'}

        url = "#{@base_url}Changes/filter"
        
        resp=formatResp({:xml=> plutora_post({:url => url, :params => opts[:searchfilter], :headers => headers})})
        # resp=plutora_post({:url => url, :params => opts[:searchfilter], :headers => headers})
        # resp=pluotra_post_json({:url => url, :params => opts[:searchfilter], :headers => headers})
        return resp
  	end

  	#-------------------------------
  	#---------- PUT CALLS ----------
  	#-------------------------------

  	#---- Update Changes By ID ----
  	def update_ChangesById(opts = {})
  		if opts[:id] == nil
  			raise "Changes By Id requires an ID value!"
  		end
  		if opts[:params] == nil
  			raise "Changes By Id requires Parameters!"
  		end

  		url = "#{@base_url}Changes/#{opts[:id]}"

  		headers = {
  			'Authorization' => "Bearer #{(opts[:token] == nil ? @token : opts[:token])}", 
  			'Accept' => 'application/json', 
  			'Content-Type' => 'application/x-www-form-urlencoded'
  			#'Content-Type' => 'application/json'
  		}

  		resp=plutora_put({:url => url, :params => opts[:params], :headers => headers})
  		return (resp.code == 204 ? true : false)
  	end

  	#---- Update Changes By Id Additional Informations ----
  	def update_ChangesById_AdditionalInformation(opts = {})
  		if opts[:id] == nil
  			raise "Changes By Id - Additional Information requires an ID value!"
  		end
  		if opts[:params] == nil
  			raise "Changes By Id - Additional Information requires Parameters!"
  		end

  		url = "#{@base_url}Changes/#{opts[:id]}/additionalInformation"

  		headers = {
  			'Authorization' => "Bearer #{(opts[:token] == nil ? @token : opts[:token])}",
  			'Content-Type' => 'application/json'
  		}

		resp=plutora_put({:url => url, :params => opts[:params], :headers => headers})
		return (resp.code == 204 ? true : false)
  	end

  	#--------------------------------
  	#---------- POST CALLS ----------
  	#--------------------------------

  	#---- Create Change ----
  	#TODO: Validate functionality
  	def create_Change(opts = {})
  		if opts[:params] == nil
  			raise "Create Change requires Parameters"
  		end

  		url = "#{@base_url}Changes"

  		headers = {
  			'Authorization' => "Bearer #{(opts[:token] == nil ? @token : opts[:token])}", 
  			'Accept' => 'application/json', 
  			'Content-Type' => 'application/x-www-form-urlencoded'
  		}

		resp=plutora_post({:url => url, :params => opts[:params], :headers => headers})
		return (resp.code == 204 ? true : false)
  	end

  	#-----------------------------
  	#---------- Helpers ----------
  	#-----------------------------

  	#---- Returns the Additional Information (Id) Value
  	def get_AdditionalInformation_FieldID(opts = {})
  		if opts[:fieldname] == nil
			raise "get Additional Information (Field ID) options variable 'fieldname' requires a value."
		end

		sId = nil

		if opts[:data].class == Hash
			if opts[:data]['Name'] == opts[:fieldname]
				sID = opts[:data]['Id']
			end
		else
			opts[:data].each do |fields|
				if fields['Name'] == opts[:fieldname]
					sID = fields['Id']
				end
			end
		end

		return (sID == nil ? false : sID)
  	end

  	#---- Returns the Additional Information (fields) Value
  	def get_AdditionalInformation_FieldValue(opts = {})
		if opts[:fieldname] == nil
			raise "get Additional Information (Field Value) options variable 'fieldname' requires a value."
		end

		sValue=nil

		if opts[:data].class == Hash
			if opts[:data]['Name'] == opts[:fieldname]
				case opts[:data]['DataType']
				when 'DatePicker'
					if !opts[:data]['Date'].class == Hash
						sValue = opts[:data]['Date']
					end
				when 'ListField'
					if opts[:data]['ListItem'].class == Hash
						if opts[:data]['ListItem'].size > 1
							sValue = opts[:data]['ListItem']['Value']
						end
					end
				when 'FreeText'
					if opts[:data]['Text'].class == String
						sValue = opts[:data]['Text']
					end
				when 'TimePicker'
					if !opts[:data]['Time'].class == Hash
						sValue = opts[:data]['Time']
					end
				when 'Number'
					if opts[:data]['Number'].class == String
						sValue = opts[:data]['Number']
					end
				when 'DateTimePicker'
					if !opts[:data]['DateTime'].class == Hash
						sValue = opts[:data]['DateTime']
					end
				when 'ListSelect'
					if !opts[:data]['MultilListItem'].class == Hash
						sValue = opts[:data]['MultilListItem']
					end
				else
					sValue=nil
				end
			end
		else
			opts[:data].each do |items|
				if items['Name'] == opts[:fieldname]
					case items['DataType']
					when 'DatePicker'
						if !items['Date'].class == Hash
							sValue = items['Date']
						end
					when 'ListField'
						if items['ListItem'].class == Hash
							if items['ListItem'].size > 1
								sValue = items['ListItem']['Value']
							end
						end
					when 'FreeText'
						if items['Text'].class == String
							sValue = items['Text']
						end
					when 'TimePicker'
						if !items['Time'].class == Hash
							sValue = items['Time']
						end
					when 'Number'
						if items['Number'].class == String
							sValue = items['Number']
						end
					when 'DateTimePicker'
						if !items['DateTime'].class == Hash
							sValue = items['DateTime']
						end
					when 'ListSelect'
						if !items['MultilListItem'].class == Hash
							sValue = items['MultilListItem']
						end
					else
						sValue=nil
					end
					break
				end
			end
		end

		return (sValue == nil ? false : sValue)
	end

	#---- Returns the System names of the indicated System Role Type
  	def get_Systems_ByRoleType(opts = {})
		if opts[:roletype] == nil
			raise "get Systems By Role Type requires a Role Type value."
		end

		sSystems=''

		if opts[:data].class == Hash
			if opts[:data]['SystemRoleType'] == opts[:roletype]
				sSystems= opts[:data]['System']
			end
		else
			opts[:data].each do |systems|
				if systems['SystemRoleType'] == opts[:roletype]
					sSystems.concat(";#{systems['System']}")
				end
			end
			sSystems = sSystems[1..-1]
		end

		if sSystems == ''
			return false
		else
			return sSystems
		end
	end

  end
end